﻿#include <Arduino.h>
#include <BLEMidi.h>

#include <arduino-timer.h>

#define A0 16
#define A1 17
#define A2 18

#define IN0 36
#define IN1 39
#define IN2 34
#define IN3 35
#define IN4 32
#define IN5 33

#define PUSH 7

#define DEBUG_FLAG 6
#define RED   4
#define GREEN 2

const unsigned inputs[6] = {IN0, IN1, IN2, IN3, IN4, IN5};

const unsigned char SteirischeBEsAsDes_push[4][13] =
    {{46,50,53,58,62,65,70,74,77,82,86,89,94},  
     {51,55,58,63,67,70,75,79,82,87,91,94, 0},  
     { 0,56,60,63,68,72,75,80,84,87,92,96, 0},  
     { 0,61,65,68,73,77,80,85,89,92,97, 0, 0}
    };   
const unsigned char SteirischeBEsAsDes_pull[4][13] =
    {{51,53,57,60,63,67,69,72,75,79,81,84,87},  
     {56,58,62,65,68,70,74,77,80,84,86,89, 0},  
     { 0,61,63,67,70,73,75,79,82,85,89,91, 0},  
     { 0,66,68,72,75,78,80,84,87,90,94, 0, 0}   
    };

struct TKey
{
  unsigned char sense; // if 0 sense active
  bool on;
  bool changed; 
};

volatile TKey keys[4][13];
volatile TKey push;

struct TCode
{
  signed char row, index;
};

const TCode codes[6][8] =
  {
  {{0, 0}, {0, 1}, {0, 2}, {0, 3}, {0, 4}, {0, 5}, {0, 6}, {0, 7}},
  {{1, 0}, {1, 1}, {1, 2}, {1, 3}, {1, 4}, {1, 5}, {1, 6}, {1, 7}},
  {{2, 0}, {2, 1}, {2, 2}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {2, 7}},
  {{3, 0}, {3, 1}, {3, 2}, {3, 3}, {3, 4}, {3, 5}, {3, 6}, {3, 7}},
  {{0, 0}, {0, 1}, {0, 2}, {0, 3}, {0, 4}, {0, 5}, {0, 6}, {0, 7}},
  {{1, 0}, {1, 1}, {1, 2}, {1, 3}, {1, 4}, {1, 5}, {-1, 0}, {-1, 0}}
  };

bool SetKey(volatile TKey & key, bool On)
{
  if (key.sense > 0)
    key.sense--;
  else 
  if (key.on != On && !key.changed)
  {
    key.on = On;
    key.sense = 20;
    key.changed = true;
  }
  return key.changed;
}

//auto timer = timer_create_default();
Timer<1, micros> timer;

bool mux_timer(void *)
{
  static unsigned char mux_state = 0;
  bool pin;
  TCode code;

  digitalWrite(DEBUG_FLAG, 1);

  if (SetKey(push, PUSH))
    push.sense = 8*push.sense;
  if (!push.changed)
  {
    for (int i = 0; i < 5; i++)
    {
      pin = digitalRead(inputs[i]);
      code = codes[i][mux_state & 7];
      if (code.row >= 0)
        SetKey(keys[code.row][code.index], pin);
    }
  }

  digitalWrite(A0, mux_state & 1);
  digitalWrite(A1, mux_state & 2);
  digitalWrite(A2, mux_state & 4);
  mux_state++;

  digitalWrite(DEBUG_FLAG, 0);

  return true; // repeat
}


void setup() 
{
  pinMode(GREEN, OUTPUT); // grün
  pinMode(RED, OUTPUT);   // rot
  Serial.begin(115200);   
  BLEMidiServer.begin("Basic Midi device"); 

  Serial.println("setup"); 

  pinMode(A0, OUTPUT);
  pinMode(A1, OUTPUT);
  pinMode(A2, OUTPUT);
  pinMode(DEBUG_FLAG, OUTPUT);

  pinMode(IN0, INPUT);
  pinMode(IN1, INPUT);
  pinMode(IN2, INPUT);
  pinMode(IN3, INPUT);
  pinMode(IN4, INPUT);
  pinMode(IN5, INPUT);
  pinMode(PUSH, INPUT);

  // call print_message in 2 seconds, but with microsecond resolution
  timer.every(1000, mux_timer);
}

bool connected = false;

void SwitchTogglePush(bool on, bool push)
{
  for (int row = 0; row < 4; row++)
    for (int index = 0; index < 13; index++)
      if (keys[row][index].on)
      {
        unsigned char pitch;
        if (push)
          pitch = SteirischeBEsAsDes_push[row][index];
        else
          pitch = SteirischeBEsAsDes_pull[row][index];
        if (pitch && connected)
        {
          if (on)
            BLEMidiServer.noteOn(row + 1, pitch, 64);
          else
            BLEMidiServer.noteOff(row + 1, pitch, 64);
        }
      }
}

void TogglePush()
{
  if (push.changed)
  { 
    SwitchTogglePush(false, !push.on);
    // b7 1f 00/01
    if (connected)
      BLEMidiServer.controlChange(7, 0x1f, push.on ? 1 : 0);  
    SwitchTogglePush(true, push.on);
    push.changed = false;
  }
}

void loop() 
{
  bool isConnected = BLEMidiServer.isConnected();
  digitalWrite(GREEN, isConnected);
  if (isConnected != connected)
  {
    if (connected)
      Serial.println("wifi connection is broken");
    else
      Serial.println("wifi is connected");
    connected = !connected;
    if (connected)
    {
      delay(100);
      for (int i = 1; i <= 6; i++)
        BLEMidiServer.programChange(i, 21); // Akkordeon
    }
  }

  TogglePush();
  bool Push = push.on;
  if (!push.changed)
  {
    for (int row = 0; row < 4; row++)
      for (int index = 0; index < 13; index++)
        if (keys[row][index].changed)
        {
          unsigned char pitch;
          if (Push)
            pitch = SteirischeBEsAsDes_push[row][index];
          else
            pitch = SteirischeBEsAsDes_pull[row][index];
          if (pitch && connected)
          {
            if (keys[row][index].on)
              BLEMidiServer.noteOn(row + 1, pitch, 127);
            else
              BLEMidiServer.noteOff(row + 1, pitch, 64);
          }
          keys[row][index].changed = false;
        }
  }
}
